package jabadot;

import javax.ejb.*;
import java.util.Date;
import java.io.*;

/** Implementation of EJB that represents one valid user.
 * JabaDot runs its own login service to allow it to have
 * full control over such things, instead of delegating
 * to the J2EE container.
 * @version $Id: UserEJB.java,v 1.1 2001/03/15 15:57:07 ian Exp $
 */
public class UserEJB implements EntityBean {

	// Database fields. Originally just a CSV-like file:
	// #name:password:fullName:email:City:Prov:Country:privs
	public String name;
	public String password;
	public String fullName;
	public String email;
	public String city;
	public String prov;
	public String country;
	public Date creationDate;
	public Date lastLoginDate;
	public boolean editPrivs = false;
	public boolean adminPrivs = false;

	protected static final int P_ADMIN = 01000;
	protected static final int P_EDIT = 0100;

	/** Construct a user with just the name */
	public String ejbCreate(String n) {
		creationDate = new Date();
		lastLoginDate = null;
		return name = n;
	}
	public void ejbPostCreate(String n) {
	}

	/** Construct a user with all text fields.
	 * After this method completes the container enters
	 * the fields we've set into the database.
	 */
	public String ejbCreate(String nick, String pw,
		String nam, String em,
		String cy, String pr, String co) {
		ejbCreate(name);			// set credt
		password = pw;
		fullName = nam;
		email = em;
		city = cy;
		prov = pr;
		country = co;
		return name;
	}
	/** Post-create for all-string version.
	 * Nothing to do.
	 */
	public void ejbPostCreate(String nick, String pw,
			String nam, String em,
		String cy, String pr, String co) {
		}

	/** Construct a user with all text fields and privs as an int. */
	public String ejbCreate(String nick, String pw,
		String nam, String em,
		String cy, String pr, String co,
		boolean edit, boolean adm) {
		//ebjCreate(nick, pw, nam, em, cy, pr, co);
				ejbCreate(nick);			// set credt
				password = pw;
				fullName = nam;
				email = em;
				city = cy;
				prov = pr;
			country = co;
			adminPrivs = adm;
			editPrivs = edit;
			return name;
	}
	public void ejbPostCreate(String nick, String pw,
			String nam, String em,
			String cy, String pr, String co,
		boolean edit, boolean adm) {
		}

	public void setName(String nick) {
		name = nick;
	}

	/** Return the nickname. */
	public String getName() {
		return name;
	}

	/** Set password */
	public void setPassword(String password) {
		this.password = password;
	}

	public String getPassword() {
		return password;
	}

	/** Validate a given password against the user's. */
	public boolean checkPassword(String userInput) {
		return password.equals(userInput);
	}

	/** Get email */
	public String getEmail() {
		return email;
	}

	/** Set email */
	public void setEmail(String email) {
		this.email = email;
	}

	/** Get fullName */
	public String getFullName() {
		return fullName;
	}

	/** Set fullName */
	public void setFullName(String fullName) {
		this.fullName = fullName;
	}

	/** Get city */
	public String getCity() {
		return city;
	}

	/** Set city */
	public void setCity(String city) {
		this.city = city;
	}

	/** Get prov */
	public String getProv() {
		return prov;
	}

	/** Set prov */
	public void setProv(String prov) {
		this.prov = prov;
	}

	/** Get country */
	public String getCountry() {
		return country;
	}

	/** Set country */
	public void setCountry(String country) {
		this.country = country;
	}

	/** Get adminPrivs */
	public boolean isAdminPrivileged() {
		return adminPrivs;
	}

	/** Set adminPrivs */
	public void setAdminPrivileged(boolean adminPrivs) {
		this.adminPrivs = adminPrivs;
	}

	/** Get EditPrivs */
	public boolean isEditPrivileged() {
		return editPrivs;
	}

	/** Set EditPrivs */
	public void setEditPrivileged(boolean editPrivs) {
		this.editPrivs = editPrivs;
	}

	/** Get the Creation Date (read only field) */
	public Date getCreationDate() {
		return creationDate;
	}

	/** Get the LastLog Date (read only field) */
	public Date getLastLoginDate() {
		return lastLoginDate;
	}

	/** Get the LastLog Date (read only field) */
	public void setLastLoginDate(Date d) {
		lastLoginDate = d;
	}

	/** Return a String representation. */
	public String toString() {
		return new StringBuffer("UserEJB[").append(name).append(',').append(fullName).append(']').toString();
	}

	// Stuff for the SessionBean interface

	// data item to hold a reference to a passed Entity context
	private EntityContext ctx;

	// save the entity context
	public void setEntityContext(EntityContext x) {
			ctx = x;
	}
	public void unsetEntityContext() {
			ctx = null;
	}
		// set the id value when ejb is activated
		public void ejbActivate() {
			name = (String)ctx.getPrimaryKey();
		}

		// nullify the ejb field values
		public void ejbPassivate() {
			name = null;
		}

		// other method implementations
		// imposed on us by interface

		public void ejbRemove() {}
		public void ejbLoad() {}
		public void ejbStore() {}
		public void ejbPostCreate() {}
}
